@@ -43,12 +43,59 @@
 # those contributions and any derivatives thereof.
 # 
 # END BPS TAGGED BLOCK }}}
+use 5.008003;
 use strict;
 use warnings; no warnings qw(redefine);
 
 package RT::Extension::MergeUsers;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
+
+=head1 NAME
+
+RT::Extension::MergeUsers - Merges two users into the same effective user
+
+=head1 DESCRIPTION
+
+This RT extension adds a "Merge Users" box to the User Administration page, 
+which allows you to merge the user you are currently viewing with another
+user on your RT instance.
+
+It also adds L</MergeInto> and L</UnMerge> functions to the L<RT::User> class,
+which allow you to programmatically accomplish the same thing from your code.
+
+It also provides a version of L<CanonicalizeEmailAddress>, which means that
+all e-mail sent from secondary users is displayed as coming from the primary
+user.
+
+=head1 INSTALLATION
+
+If you're upgrading then, as well, read L</UPGRADING> below.
+
+    perl Makefile.PL
+    make
+    make install
+    clear your mason cache
+        most often this would be rm -rf /opt/rt3/var/mason_data/*
+    restart apache
+
+For RT 3.8
+
+    Add RT::Extension::MergeUsers to your /opt/rt3/etc/RT_SiteConfig.pm file
+    Set(@Plugins, qw(RT::Extension::MergeUsers));
+
+    If you have more than one Plugin enabled, you must enable them as one 
+    Set(@Plugins, qw(Foo Bar)); command
+
+=head1 UPGRADING
+
+If you are upgrading from 0.03_01 or earlier, you must run F<rt-upgrade-merged-users>.
+This script will create MergedUsers Attributes so RT can know when you're looking
+at a user that other users have been merged into. If you don't run this script,
+you'll have issues unmerging users. It can be safely run multiple times, it will
+only create Attributes as needed.
+
+=cut
 
 package RT::User;
 
@@ -179,6 +226,9 @@ sub MergeInto {
     return (0, "User @{[$canonical_self->Name]} has already been merged")
            if defined $new and $new->Content == $canonical_self->id;
 
+    # clean the cache
+    delete $EFFECTIVE_ID_CACHE{$self->id};
+
     # do the merge
     $canonical_self->SetAttribute(
         Name => "EffectiveId",
@@ -190,11 +240,11 @@ sub MergeInto {
     $merged_users->SetContent( [$canonical_self->Id, @{$merged_users->Content}] );
 
     $canonical_self->SetComments( join "\n", grep /\S/,
-        $canonical_self->Comments,
+        $canonical_self->Comments||'',
         "Merged into ". $merge->EmailAddress ." (". $merge->id .")",
     );
     $merge->SetComments(join "\n", grep /\S/,
-        $merge->Comments,
+        $merge->Comments||'',
         $canonical_self->EmailAddress." (".$canonical_self->id.") merged into this user",
     );
     return (1, "Merged users successfuly");
@@ -206,12 +256,16 @@ sub UnMerge {
     my ($current) = $self->Attributes->Named("EffectiveId");
     return (0, "Not a merged user") unless $current;
 
+    # flush the cache, or the Sets below will
+    # clobber $self
+    delete $EFFECTIVE_ID_CACHE{$self->id};
+
     my $merge = RT::User->new($RT::SystemUser);
     $merge->Load( $current->Content );
 
     $current->Delete;
     $self->SetComments( join "\n", grep /\S/,
-        $self->Comments,
+        $self->Comments||'',
         "Unmerged from ". $merge->EmailAddress ." (".$merge->id.")",
     );
 
@@ -219,6 +273,7 @@ sub UnMerge {
         $merge->Comments,
         $self->EmailAddress ." (". $self->id .") unmerged from this user",
     );
+
     my $merged_users = $merge->GetMergedUsers;
     my @remaining_users = grep { $_ != $self->Id } @{$merged_users->Content};
     if (@remaining_users) {
@@ -227,7 +282,7 @@ sub UnMerge {
         $merged_users->Delete;
     }
 
-    return ($merge->id, "Unmerged from @{[$merge->EmailAddress]}");
+    return ($merge->id, "Unmerged @{[$self->NameAndEmail]} from @{[$merge->NameAndEmail]}");
 }
 
 sub SetEmailAddress {
@@ -249,9 +304,21 @@ sub SetEmailAddress {
     return $self->_Set( Field => 'EmailAddress', Value => $value );
 }
 
+sub NameAndEmail {
+    my $self = shift;
+    my $name = $self->Name;
+    my $email = $self->EmailAddress;
+
+    if ($name eq $email) {
+        return $email;
+    } else {
+        return "$name <$email>";
+    }
+}
+
 package RT::Users;
 use RT::Users;
-        
+
 =head2 Next
 
 This custom iterator makes sure that duplicate users are never shown in search results.
@@ -280,4 +347,14 @@ sub Next {
 
 }
 
+=head1 AUTHOR
+
+Alex Vandiver E<lt>alexmv@bestpractical.comE<gt>
+
+=head1 LICENSE
+
+GPL version 2.
+
+=cut
+
 1;
